@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 
 #
 # This file is a test harness for svn-load
@@ -156,9 +156,9 @@ class BaseLoadCase(unittest.TestCase):
     def testLoad(self):
         self.svnImport(self.importdir, self.url)
         for d in self.loaddirs:
-            self.failUnless(self.loaddir(d))
-            self.failUnless(self.exportRepository())
-            self.failUnless(self.compare_trees(d,
+            self.assertTrue(self.loaddir(d))
+            self.assertTrue(self.exportRepository())
+            self.assertTrue(self.compare_trees(d,
                             os.path.join(self.exportparent, 'export')))
 
 
@@ -231,8 +231,8 @@ class BaseMoveCase(BaseLoadCase):
         self.movemap = tempfile.mkstemp()[1]
         self.makeMoveMap()
         self.extra_args.extend(['--move-map', self.movemap])
-        self.filesmap_forward = dict(zip(self.files_old, self.files_new))
-        self.filesmap_backwards = dict(zip(self.files_new, self.files_old))
+        self.filesmap_forward = dict(list(zip(self.files_old, self.files_new)))
+        self.filesmap_backwards = dict(list(zip(self.files_new, self.files_old)))
 
     def tearDown(self):
         BaseLoadCase.tearDown(self)
@@ -278,7 +278,7 @@ class BaseMoveCase(BaseLoadCase):
             return
 
         self.svnImport(self.importdir, self.url)
-        self.failUnless(self.loaddir(self.loaddirs[0]))
+        self.assertTrue(self.loaddir(self.loaddirs[0]))
 
         svnxml = subprocess.check_output(['svn', 'log'] + self.extra_svn_args +
                                          ['-l1', '-v', '--xml', self.url])
@@ -287,16 +287,16 @@ class BaseMoveCase(BaseLoadCase):
         for e in paths:
             # Assume only paths with actions are in the log unless
             # we find a counter-example
-            self.failUnless('action' in e.attrib.keys())
+            self.assertTrue('action' in list(e.attrib.keys()))
             path = os.path.relpath(e.text, start=os.sep)
             if path in self.files_old:
                 self.assertEqual(e.attrib['action'], 'D')
             elif path in self.files_new:
                 self.assertEqual(e.attrib['action'], 'A')
-                self.failUnless('copyfrom-path' in e.attrib.keys())
+                self.assertTrue('copyfrom-path' in list(e.attrib.keys()))
                 from_path = os.path.relpath(e.attrib['copyfrom-path'],
                                             start=os.sep)
-                self.failUnless(from_path in self.files_old)
+                self.assertTrue(from_path in self.files_old)
 
 
 class SingleFileMove(BaseMoveCase):
@@ -445,6 +445,28 @@ class DirectorySymLinkToFile(BaseLoadCase):
         open(os.path.join(self.loaddirs[0], 'bar'), 'w').close()
 
 
+class DirectoryToFile(BaseLoadCase):
+    def importDirSetUp(self):
+        BaseLoadCase.importDirSetUp(self)
+        open(os.path.join(self.importdir, 'foo'), 'w').close()
+
+    def loadDirsSetUp(self):
+        BaseLoadCase.loadDirsSetUp(self)
+        os.mkdir(os.path.join(self.loaddirs[0], 'foo'))
+
+    def testLoad(self):
+        # A failure here is expected - svn does not support changing
+        # a file to a directory in a single commit. Perhaps we should
+        # support a multiple-commit mode to deal with such things, but
+        # until then we'll just test that this fails as expected.
+        # https://bugs.debian.org/573090
+        try:
+            BaseLoadCase.testLoad(self)
+        except AssertionError:
+            return
+        raise AssertionError
+
+
 class ReimportSymlinkBase(BaseLoadCase):
     def _fillDirectory(self, directory):
         # overwrite in derived class
@@ -532,15 +554,15 @@ class GlobHandling(BaseLoadCase):
         self.svnImport(self.importdir, self.url)
         os.system('%s %s / %s*' % (svnload, self.url,
                                    os.path.join(self.loaddirparent, "foo*")))
-        self.failUnless(self.exportRepository())
-        self.failUnless(
+        self.assertTrue(self.exportRepository())
+        self.assertTrue(
             self.compare_trees(self.loaddirs[-1],
                                os.path.join(self.exportparent, 'export')))
 
 
 class TempConfigFile:
     def __init__(self):
-        self.f = tempfile.NamedTemporaryFile(delete=False)
+        self.f = tempfile.NamedTemporaryFile(mode='w', delete=False)
         self.path = self.f.name
 
     def __del__(self):
@@ -551,7 +573,7 @@ class SvnServePasswdFile(TempConfigFile):
     def __init__(self, userDict):
         TempConfigFile.__init__(self)
         self.f.write('[users]\n')
-        users = userDict.keys()
+        users = list(userDict.keys())
         for u in users:
             self.f.write('%s = %s\n' % (u, userDict[u]))
         self.f.close()
@@ -607,7 +629,7 @@ class AuthenticatedServerPrompt(AuthenticatedServer):
         args = ['./svn-load', self.url, os.sep, ldir]
         args = args + self.extra_args
         child = pexpect.spawn(' '.join(args))
-        child.logfile = sys.stdout
+        child.logfile = sys.stdout.buffer
         child.expect('Username \(svn\): ')
         child.sendline('alice')
         child.expect('Password for.*: ')
